#!/usr/bin/env python

import os
import sys
import re
import commands


def readh(date, okey):

    #
    # reading the headers of all the FITS files in ../[date]/rawdata
    # and extracting twiligh flat frames based on the OBJECT name
    # and dividing them into morning and evening based on the LT
    #

    fflag = 0
    eflag = 0
    mflag = 0

    foe = open('eflat.tmp', 'w')
    fom = open('mflat.tmp', 'w')
    flist = os.listdir('../'+date+'/rawdata')
    for file in flist:
        if re.search('fits', file):
            rfile = '../'+date+'/rawdata/'+file
            f = open(rfile)
            flag = 0
            while 1:
                line = f.read(80)
                if flag == 0:
                    if re.match('NAXIS1', line):
                        flag = 1
                else:
                    if re.match('OBJECT', line):
                        object = (line[:-1].split())[2][1:]
                    elif re.match('TIME_LT', line):
                        timelthh = int((line[:-1].split())[2][1:3])
                    elif  re.match('END', line):
                        break
            f.close()

            if object == okey:
                fflag += 1
                if timelthh > 15 and timelthh < 22:
                    print >> foe, file
                    eflag += 1
                if timelthh > 2 and timelthh < 9:
                    print >> fom, file
                    mflag += 1

    foe.close()
    fom.close()

    return fflag, eflag, mflag



def bkcheck(band, mode, date, numarr, lvlarr):

    flag = 0
    fw = open('mktwfwarning.txt', 'aw')
    fo = open(band+date+mode+'.dat', 'w')
    print >> fo, numarr[0], lvlarr[0] 
    for i in range(1, len(numarr)):
        print >> fo, numarr[i], lvlarr[i], 
        if lvlarr[i] - lvlarr[i-1] < 0 :
            flag += 1
            print 'background level unstable at  %s%s_%s ' % (band, date, numarr[i-1])
            print >> fw, 'background level unstable at  %s%s_%s ' % (band, date, numarr[i-1])
            print >> fo, '*'
        else:
            print >> fo, ''
    fo.close()
    fw.close()

    return flag



def pickpairs(band, mode, date, numarr, lvlarr, skydiff):

    #
    # reversing the order to pick pairs from frames with a higher background level
    #
    numarr.reverse()
    lvlarr.reverse()

    num = 0
    fo = open(band+'flatlist.0', 'aw')
    for i in range(len(numarr)):
        for j in range(i+1, len(numarr)):
            if lvlarr[i] - lvlarr[j] > skydiff:
                num += 1
                print >> fo, date, numarr[i], numarr[j], lvlarr[i], lvlarr[j]
                break
    fo.close()

    return num


def mklist(band, mode, date, skydiff, maxgood):

    # 
    # fdict = {'frame number(str)':backgroundlevel(float)}
    #
    fdict = {}
    f = open('twflat.tmp')
    for line in f:
        v = line[:-1].split()
        num = v[0][8:12]
        if v[0][0] == band:
            fdict[num] = float(v[1])
    f.close()

    if mode == 'e':
        vbool = True
    elif mode == 'm':
        vbool = False

    numarr = [] # frame number
    lvlarr = [] # background level

    #
    # the frames are sorted so that a twilight free frame comes first 
    # saturated frames are excluded 
    #
    for k, v in sorted(fdict.items(), reverse=vbool):
        if v < maxgood:
            numarr.append(k)
            lvlarr.append(v)
        else:
            break

    # check if the background level is monotonically increasing or decreasing 
    bflag = bkcheck(band, mode, date, numarr, lvlarr)    

    # pick pairs of frames and write to band+'flatlist'
    num = pickpairs(band, mode, date, numarr, lvlarr, skydiff)

    return num 


def listofaday(date, okey, skydiff, maxgood, bandarr, tnum):

    SCRIPTDIR = os.path.realpath(os.path.dirname(sys.argv[0]))
    cprog = 'twflatlev'

    #
    # reading the FITS header of the rawdata of the day
    # and counting the number of twilight flat frames 
    # and creating eflat.tmp and mflat.tmp
    #
    fflag, eflag, mflag = readh(date, okey)

    #
    # fflag : the total number of twilight flat frames
    # eflag : the total number of the evening twilight flat frames
    # mflag : the total number of the morning twilight flat frames
    #

    if fflag == 0:
        print 'No flat frames taken on %s' % date
        os.remove('eflat.tmp')
        os.remove('mflat.tmp')
        return tnum

    if eflag > 0:
        os.system('%s/%s  %s eflat.tmp' % (SCRIPTDIR, cprog, date))
        for band in bandarr:
            num = mklist(band, 'e', date, skydiff, maxgood)
            tnum[band] += num

    if mflag > 0:
        os.system('%s/%s %s mflat.tmp' % (SCRIPTDIR, cprog, date))
        for band in bandarr:
            num = mklist(band, 'm', date, skydiff, maxgood)
            tnum[band] += num

    os.remove('eflat.tmp')
    os.remove('mflat.tmp')
    os.remove('twflat.tmp')

    return tnum


def exslope(band):

    SCRIPTDIR = os.path.realpath(os.path.dirname(sys.argv[0]))
    cprog = 'exslopefz'

    snum = 0
    fo = open(band+'flatlist', 'w')
    f = open(band+'flatlist.0')
    for line in f:
        v = line[:-1].split()
        fits1 = '../'+v[0]+'/rawdata/'+band+v[0]+'_'+v[1]+'.fits'
        fits2 = '../'+v[0]+'/rawdata/'+band+v[0]+'_'+v[2]+'.fits'
        if os.path.exists(fits1) and os.path.exists(fits2):
            file1 = fits1
            file2 = fits2
        elif os.path.exists(fits1+'.fz') and os.path.exists(fits2+'.fz'):
            file1 = fits1+'.fz[1]'
            file2 = fits2+'.fz[1]'
        elif os.path.exists(fits1+'.ic') and os.path.exists(fits2+'.ic'):
            file1 = fits1+'.ic[1]'
            file2 = fits2+'.ic[1]'
        else:
            print 'Something wrong with compression for %s and %s' % (fits1, fits2)
            print 'ic or fz?'
            continue

        flag = commands.getoutput('%s/%s %s %s' % (SCRIPTDIR, cprog, file1, file2))
        if flag == '1':
            snum += 1
            print >> fo, line[:-1]

    f.close()
    fo.close()

    if snum == 0:
        print 'There is no good flat frame for %s-band.' % band
    else:
        print 'a total of %d pairs are selected for %s' % (snum, band)

    
    
if __name__ == "__main__":

    argvs = sys.argv
    argc = len(argvs)

    skydiff = 2000.
    maxgood = 6000.  
    exflag = 0
    okey = 'twflat'
    bandarr = ['j', 'h', 'k']

    if argc < 2:
        print 'Usage # mktwflist.py date-list'
        print 'option'
        print '-noexgap'
        print '-skydiff=float (default 2000)'
        print '-maxgood=float (default 6000)'
        print '-keyword=string (default twflat)'
        print '-band=string (ex. -band=j -band=h,k)'
        sys.exit()

    oanum = 1
    for item in argvs:
        if item == '-noexgap':
            exflag = -99
            oanum += 1
        elif item.startswith('-skydiff'):
            skydiff = float(item[9:])
            oanum += 1
        elif item.startswith('-maxgood'):
            maxgood = float(item[9:])
            oanum += 1
        elif item.startswith('-keyword'):
            okey = item[9:]
            oanum += 1
        elif item.startswith('-band'):
            bandarr = item[6:].split(',')
            oanum += 1

    if oanum == argc:
        print 'Input the list'
        print 'Usage # mktwflist.py date-list +options'
        sys.exit()

    tnum = {}
    for band in bandarr:
        tnum[band] = 0

    f = open(argvs[1])
    for line in f:
        date = line[:-1]
        if re.search('\S', date):
            print date
            tnum = listofaday(date, okey, skydiff, maxgood, bandarr, tnum)
    f.close()

    #
    # excluding pairs with the gap at the central column
    #
    if exflag == 0:
        for band in bandarr:
            if tnum[band] == 0:
                print 'No frame for %s-band flat' % band 
            else:
                exslope(band)
    else:
        for band in bandarr:
            os.rename(band+'flatlist.0', band+'flatlist')
            print 'a total of %d pairs are selected for %s' % (tnum[band], band)


    print 'Done'




